/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    acmFoam

Description
    Transient solver for incompressible, laminar flow of Newtonian fluids.

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "singlePhaseTransportModel.H"
#include "RASModel.H"
#include "acmControl.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMesh.H"
#   include "createFields.H"
#   include "initContinuityErrs.H"

#   include "readTimeControls.H"
#   include "CourantNo.H"
#   include "setInitialDeltaT.H"

acmControl acm(mesh);

dimensionedScalar betaSqrEps(acm.dict().lookup("betaSqrEps"));
scalar gamma0(readScalar(acm.dict().lookup("gamma0")));


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nStarting time loop\n" << endl;

    while (acm.loop())
    {
        Info<< "Time = " << runTime.timeName() << nl << endl;

#       include "readTimeControls.H"
#       include "CourantNo.H"
#       include "setDeltaT.H"

        fvVectorMatrix UEqn
        (
            fvm::ddt(U)
          + fvm::div(phi, U)
          + turbulence->divDevReff(U)
        );

        solve(UEqn == -fvc::grad(p));

        phi = (fvc::interpolate(U) & mesh.Sf());

        adjustPhi(phi, U, p);

        scalar tc = 1;
        dimensionedScalar
        gamma
        (
            "gamma",
            dimensionSet(0,0,-1,0,0),
            gamma0*Foam::exp(tc-runTime.value())
        );
        
        while (acm.correctNonOrthogonal())
        {
            fvScalarMatrix pEqnAcm
            (    
                betaSqrEps*sqr(eps)*fvm::ddt(p) 
                ==
              - fvc::div(phi)
              - gamma*betaSqrEps*sqr(eps)*p
            );

            pEqnAcm.setReference(pRefCell, pRefValue);

            pEqnAcm.solve();
        }

#       include "continuityErrs.H"

        turbulence->correct();

        runTime.write();

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;
    }

    Info<< "End\n" << endl;

    return 0;
}


// ************************************************************************* //
